
/*

  Regenerage a tape file (including variable block counts) from a
  representation in an ordinary file created by `tape2file'.

  Representation is: each block of data to be preceded by an int, denoting
  the block size.

*/

#include <stdio.h>
#include <fcntl.h>

#define MAX_RECORD_SIZE 80000

static char *usage        = "USAGE: file2tape <file> <device>\n" ;

extern int  errno, sys_nerr ;
extern char *sys_errlist [] ;

void get_params (file, device, argc, argv)

char **device, **file ;
int argc ;
char *argv [] ;

{
  if (argc != 3)
  {
    printf (usage) ;
    exit (1) ;
  }
  *file   = argv [1] ;
  *device = argv [2] ;
}

void sys_assert (condition, usr_message)
/*

  Test `condition'.  Abort program, reporting `usr_message', if it fails.

*/
int condition ;
char *usr_message ;
{
  if (!condition)
  {
    fprintf (stderr, "ERROR: %s", usr_message) ;
    if ((errno > 0) && (errno < sys_nerr))
      fprintf (stderr, "... %s", sys_errlist [errno]) ;
    fprintf (stderr, ".\n") ;
    exit (1) ;
  }
}

void main (argc, argv)
int argc ;
char *argv [] ;
{
  char err_mess [128] ;

  char *device, *file ;
  int src_desc, dst_desc ;
  int going = 1 ;

  get_params (&file, &device, argc, argv) ;

  src_desc = open (file, O_RDONLY) ;
  strcpy (err_mess, "Failed to open \"") ;
  strcat (err_mess, file) ;
  strcat (err_mess, "\"") ;
  sys_assert (src_desc >= 0, err_mess) ;

  dst_desc = open (device, O_CREAT | O_TRUNC | O_WRONLY, 0x1a4) ;
  strcpy (err_mess, "Failed to open \"") ;
  strcat (err_mess, device) ;
  strcat (err_mess, "\"") ;
  sys_assert (dst_desc >= 0, err_mess) ;

  while (going)
  {
    char record [MAX_RECORD_SIZE] ;

    int result, len ;

    result = read (src_desc, (char*) &len, sizeof (int)) ;
    sys_assert (result >= 0, "Read failed") ;
    if (result == 0)
    {
      fprintf (stdout, "EOF encountered\n") ;
      going = 0 ;
    }
    else
    {
      result = read (src_desc, record, len) ;
      sys_assert (result >= 0, "Read failed") ;

      /*
      printf ("len = %d\n", len) ;
      */
      result = write (dst_desc, record, len) ;
      sys_assert (result >= 0, "Write failed") ;
    }
  }
  close (dst_desc) ;
  close (src_desc) ;
  exit (0) ;
}
